/* global Fapi,Fql */

/**
 * Objet permettant de stocker des éléments récupérés par l'API Ajax * 
 * 
 * @constructor
  */
Fapi.Cache = function () {
    this.configMap = new Map();
    this.thesaurusMap = new Map();
    this.fichesMap = new Map();
};


/**
 * 
 * Rétrocompatibilité
 */
Fapi.Wrapper = Fapi.Cache;

Fapi.Cache.prototype.getFichothequeConfig = function (name) {
    return this.configMap.get(name);
};


/**
 * Retourne le thésaurus correspondant à la clé en argument
 * 
 * @param {string} key Clé du thésaurus 
 * @returns {Fapi.Cache.Thesaurus} Objet Thesaurus
 */
Fapi.Cache.prototype.getThesaurus = function (key) {
    return this.thesaurusMap.get(key);
};


/**
 * Retourne l'objet Fiches correspondant à la clé en argument
 * 
 * @param {string} key Clé d'un ensemble fiches
 * @returns {Fapi.Cache.Fiches} Objet Fiches
 */
Fapi.Cache.prototype.getFiches = function (key) {
    return this.fichesMap.get(key);
};

Fapi.Cache.prototype.putConfig = function (name, fichothequeUrl, options) {
    var configMap = this.configMap;
    if (configMap.has(name)) {
        let currentConfig = configMap.get(name);
        if (fichothequeUrl) {
            currentConfig.fichothequeUrl = fichothequeUrl;
        }
        Fapi.Config.assignOptions(currentConfig.options, options);
        return currentConfig;
    } else {
        let newConfig = new Fapi.Config(fichothequeUrl, options);
        configMap.set(name, newConfig);
        return newConfig;
    }
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {Fapi.Config} apiConfig Configuration à utiliser
 * @param {string} thesaurusName Nom du thésaurus
 * @param {Object} options Options de l'appel
 * @param {string} options.thesaurusKey Clé du thésaurus à utiliser au lieu du nom du thésaurus
 * @param {string} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @returns {function} Fonction d'appel de l'API
 */
Fapi.Cache.prototype.initThesaurus = function (apiConfig, thesaurusName, options) {
    var propertiesMapping;
    var requestParameters = {};
    var thesaurusMap = this.thesaurusMap;
    var thesaurusKey = thesaurusName;
    var requestParameters = {};
    if (options) {
        if (options.thesaurusKey) {
            thesaurusKey = options.thesaurusKey;
        }
        if (options.propertiesMapping) {
            propertiesMapping = options.propertiesMapping;
        }
        if (options.requestParameters) {
            Object.assign(requestParameters, options.requestParameters);
        }
    }
    if (!propertiesMapping) {
        propertiesMapping = apiConfig.options.propertiesMapping;
    }
    return function (callback) {
        Fapi.Ajax.loadThesaurus(apiConfig, thesaurusName, {
            propertiesMapping: propertiesMapping,
            requestParameters: requestParameters,
            callback: function (data) {
                let thesaurus = new Fapi.Cache.Thesaurus(apiConfig, options, data);
                thesaurusMap.set(thesaurusKey, thesaurus);
                if (callback) {
                    callback();
                }
            }
        });
    };
};


/**
 * Retourne une fonction d'appel de l'API, cette fonction prend comme argument
 * une fonction de retour pour permettre le chainage 
 * 
 * @param {Fapi.Config} apiConfig Configuration à utiliser
 * @param {string} fichesKey Clé à utiliser pour conserver le résultat
 * @param {Object} options Options de l'appel
 * @param {Object} options.propertiesMapping Nom de l'export tabulaire à utiliser pour définir les propriétés d'un mot-clé
 * @param {Fql.FicheQuery} options.Query Instance de Fql.FicheQuery indiquant les fiches à récupérer
 * @param {Object} options.ql Objet correspondant à un objet Options utilisé en argument du constructeur de Fql.FicheQuery 
 * @returns {function} Fonction d'appel de l'API
 */
Fapi.Cache.prototype.initFiches = function (apiConfig, fichesKey, options) {
    var propertiesMapping;
    var requestParameters = {};
    var fichesMap = this.fichesMap;
    var requestParameters = {};
    var queries = null;
    if (options) {
        if (options.propertiesMapping) {
            propertiesMapping = options.propertiesMapping;
        }
        if (options.queries) {
            queries = options.queries;
        }
        if (options.requestParameters) {
            Object.assign(requestParameters, options.requestParameters);
        }
        if (options.sort) {
            requestParameters.sort = options.sort;
        }
    }
    if (!queries) {
        queries = new Fql.FicheQuery({
            corpus: fichesKey
        });
    }
    return function (callback) {
        Fapi.Ajax.loadFiches(apiConfig, queries, {
            propertiesMapping: propertiesMapping,
            requestParameters: requestParameters,
            callback: function (data) {
                let fiches = new Fapi.Cache.Fiches(data);
                fichesMap.set(fichesKey,fiches);
                if (callback) {
                    callback();
                }
            }
        });
    };
};

/*
 * {
 *  fichotheques:  [
    *  {
    *      name: "",
    *      url: "",
    *      options: {
    *          version: "action",
    *          propertiesMapping: ""
    *      },
    *      thesaurus: [
              {
                name: "",
                options: {

                }
              }
    *      ]
    *  }
 *  ]
 * }
 */

Fapi.Cache.init = function (initObject, callback) {
    var apiCache = new Fapi.Cache();
    var thesaurusInitArray = [];
    var callbackDone = false;
    if (initObject.fichotheques) {
        for(let fichotheque of initObject.fichotheques) {
            let fichothequeName = fichotheque.name;
            let fichothequePrefix = "";
            if ((fichothequeName) && (fichothequeName !== "_default") && (fichothequeName !== "_current")) {
                fichothequePrefix = fichotheque.name + "/";
            }
            let apiConfig = apiCache.putConfig(fichothequeName, fichotheque.url, fichotheque.options);
            if (fichotheque.thesaurus) {
                for(let thesaurus of fichotheque.thesaurus) {
                    if (typeof thesaurus === "string") {
                        thesaurusInitArray.push({
                            name: thesaurus,
                            options: {
                                thesaurusKey: fichothequePrefix + thesaurus
                            },
                            config: apiConfig,
                            done: false
                        });
                    } else {
                        let name = thesaurus.name;
                        if (name) {
                            let options = thesaurus.options;
                            if (!options) {
                                options = {
                                    thesaurusKey: fichothequePrefix + name
                                }
                            } else if (!options.thesaurusKey) {
                                options = Object.assign({
                                    thesaurusKey: fichothequePrefix + name
                                }, options);
                            }
                            thesaurusInitArray.push({
                                name: name,
                                options: options,
                                config: apiConfig,
                                done: false
                            });
                        }
                    }
                }
            }
        }
    }
    if (thesaurusInitArray.length > 0) {
        for(let thesaurusInit of thesaurusInitArray) {
            let initFunction = apiCache.initThesaurus(thesaurusInit.config, thesaurusInit.name, thesaurusInit.options);
            initFunction(function () {
               thesaurusInit.done = true;
               _checkThesaurusInit();
            });
        }
    } else {
        callback(apiCache);
    }
    
    function _checkThesaurusInit() {
        let allDone = true;
        for(let thesaurusInit of thesaurusInitArray) {
            if (!thesaurusInit.done) {
                allDone = false;
                break;
            }
        }
        if ((allDone) && (!callbackDone)) {
            callbackDone = true;
            callback(apiCache);
        }
    }

};
